@@ -1,4 +1,6 @@
 import copy
+import operator
+from functools import cached_property, reduce
 from typing import Dict, List, Optional
 
 import torch
@@ -16,6 +18,7 @@ from ai_traineree.loggers import DataLogger
 from ai_traineree.networks.bodies import ActorBody, CriticBody
 from ai_traineree.noise import GaussianNoise
 from ai_traineree.types import ActionType, AgentState, BufferState, DoneType, NetworkState, ObsType, RewardType
+from ai_traineree.types.dataspace import DataSpace
 from ai_traineree.utils import to_numbers_seq, to_tensor
 
 
@@ -24,15 +27,19 @@ class DDPGAgent(AgentBase):
     Deep Deterministic Policy Gradients (DDPG).
 
     Instead of popular Ornstein-Uhlenbeck (OU) process for noise this agent uses Gaussian noise.
+
+    This agent is intended for continuous tasks.
     """
 
-    name = "DDPG"
+    model = "DDPG"
 
-    def __init__(self, obs_size: int, action_size: int, noise_scale: float=0.2, noise_sigma: float=0.1, **kwargs):
+    def __init__(
+        self, obs_space: DataSpace, action_space: DataSpace, noise_scale: float=0.2, noise_sigma: float=0.1, **kwargs
+    ):
         """
         Parameters:
-            obs_size: Number of input dimensions.
-            action_size: Number of output dimensions
+            obs_space (DataSpace): Dataspace describing the input.
+            action_space (DataSpace): Dataspace describing the output.
             noise_scale (float): Added noise amplitude. Default: 0.2.
             noise_sigma (float): Added noise variance. Default: 0.1.
 
@@ -49,34 +56,32 @@ class DDPGAgent(AgentBase):
             warm_up (int): Number of samples to observe before starting any learning step. Default: 0.
             update_freq (int): Number of steps between each learning step. Default 1.
             number_updates (int): How many times to use learning step in the learning phase. Default: 1.
-            action_min (float): Minimum returned action value. Default: -1.
-            action_max (float): Maximum returned action value. Default: 1.
-            action_scale (float): Multipler value for action. Default: 1.
 
         """
         super().__init__(**kwargs)
         self.device = self._register_param(kwargs, "device", DEVICE)
-        self.obs_size = obs_size
-        self.action_size = action_size
-        self._config['obs_size'] = self.obs_size
-        self._config['action_size'] = self.action_size
-        obs_shape = (obs_size,)
-        action_shape = (action_size,)
+        self.obs_space = obs_space
+        self.action_space = action_space
+        self._config['obs_space'] = self.obs_space
+        self._config['action_space'] = self.action_space
+
+        action_shape = action_space.to_feature()
+        action_size = reduce(operator.mul, action_shape)
 
         # Reason sequence initiation.
         hidden_layers = to_numbers_seq(self._register_param(kwargs, 'hidden_layers', (64, 64)))
         self.actor = ActorBody(
-            obs_shape, action_shape, hidden_layers=hidden_layers, gate_out=torch.tanh).to(self.device)
+            obs_space.shape, action_shape, hidden_layers=hidden_layers, gate_out=torch.tanh).to(self.device)
         self.critic = CriticBody(
-            obs_shape, action_size, hidden_layers=hidden_layers).to(self.device)
+            obs_space.shape, action_size, hidden_layers=hidden_layers).to(self.device)
         self.target_actor = ActorBody(
-            obs_shape, action_shape, hidden_layers=hidden_layers, gate_out=torch.tanh).to(self.device)
+            obs_space.shape, action_shape, hidden_layers=hidden_layers, gate_out=torch.tanh).to(self.device)
         self.target_critic = CriticBody(
-            obs_shape, action_size, hidden_layers=hidden_layers).to(self.device)
+            obs_space.shape, action_size, hidden_layers=hidden_layers).to(self.device)
 
         # Noise sequence initiation
         self.noise = GaussianNoise(
-            shape=(action_size,), mu=1e-8, sigma=noise_sigma, scale=noise_scale, device=self.device)
+            shape=action_shape, mu=1e-8, sigma=noise_sigma, scale=noise_scale, device=self.device)
 
         # Target sequence initiation
         hard_update(self.target_actor, self.actor)
@@ -89,9 +94,6 @@ class DDPGAgent(AgentBase):
         self.critic_optimizer = Adam(self.critic.parameters(), lr=self.critic_lr)
         self.max_grad_norm_actor = float(self._register_param(kwargs, "max_grad_norm_actor", 10.0))
         self.max_grad_norm_critic = float(self._register_param(kwargs, "max_grad_norm_critic", 10.0))
-        self.action_min = float(self._register_param(kwargs, 'action_min', -1))
-        self.action_max = float(self._register_param(kwargs, 'action_max', 1))
-        self.action_scale = float(self._register_param(kwargs, 'action_scale', 1))
 
         self.gamma = float(self._register_param(kwargs, 'gamma', 0.99))
         self.tau = float(self._register_param(kwargs, 'tau', 0.02))
@@ -135,6 +137,14 @@ class DDPGAgent(AgentBase):
             and self.buffer == o.buffer \
             and self.get_network_state() == o.get_network_state()
 
+    @cached_property
+    def action_min(self):
+        return to_tensor(self.action_space.low)
+
+    @cached_property
+    def action_max(self):
+        return to_tensor(self.action_space.high)
+
     @torch.no_grad()
     def act(self, obs: ObsType, noise: float=0.0) -> List[float]:
         """Acting on the observations. Returns action.
@@ -149,7 +159,7 @@ class DDPGAgent(AgentBase):
         t_obs = to_tensor(obs).float().to(self.device)
         action = self.actor(t_obs)
         action += noise*self.noise.sample()
-        action = torch.clamp(action*self.action_scale, self.action_min, self.action_max)
+        action = torch.clamp(action, self.action_min, self.action_max)
         return action.cpu().numpy().tolist()
 
     def step(self, obs: ObsType, action: ActionType, reward: RewardType, next_obs: ObsType, done: DoneType) -> None:
@@ -165,7 +175,7 @@ class DDPGAgent(AgentBase):
 
     def compute_value_loss(self, states, actions, next_states, rewards, dones):
         next_actions = self.target_actor.act(next_states)
-        assert next_actions.shape == actions.shape
+        assert next_actions.shape == actions.shape, f"{next_actions.shape} != {actions.shape}"
         Q_target_next = self.target_critic.act(next_states, next_actions)
         Q_target = rewards + self.gamma * Q_target_next * (1 - dones)
         Q_expected = self.critic(states, actions)
@@ -186,11 +196,12 @@ class DDPGAgent(AgentBase):
         rewards = to_tensor(experiences['reward']).float().to(self.device).unsqueeze(1)
         dones = to_tensor(experiences['done']).type(torch.int).to(self.device).unsqueeze(1)
         states = to_tensor(experiences['state']).float().to(self.device)
-        actions = to_tensor(experiences['action']).to(self.device)
+        actions = to_tensor(experiences['action']).float().to(self.device).view((-1,) + self.action_space.shape)
         next_states = to_tensor(experiences['next_state']).float().to(self.device)
-        assert rewards.shape == dones.shape == (self.batch_size, 1)
-        assert states.shape == next_states.shape == (self.batch_size, self.obs_size)
-        assert actions.shape == (self.batch_size, self.action_size)
+
+        assert rewards.shape == dones.shape == (self.batch_size, 1), f"R.shape={rewards.shape}, D.shap={dones.shape}"
+        assert states.shape == next_states.shape == (self.batch_size,) + self.obs_space.shape, f"states.shape: {states.shape}"
+        assert actions.shape == (self.batch_size,) + self.action_space.shape, f"actions.shape: {actions.shape}"  # type: ignore
 
         # Value (critic) optimization
         loss_critic = self.compute_value_loss(states, actions, next_states, rewards, dones)
@@ -245,9 +256,9 @@ class DDPGAgent(AgentBase):
 
     def get_state(self) -> AgentState:
         return AgentState(
-            model=self.name,
-            obs_space=self.obs_size,
-            action_space=self.action_size,
+            model=self.model,
+            obs_space=self.obs_space,
+            action_space=self.action_space,
             config=self._config,
             buffer=copy.deepcopy(self.buffer.get_state()),
             network=copy.deepcopy(self.get_network_state()),
@@ -265,7 +276,7 @@ class DDPGAgent(AgentBase):
     @staticmethod
     def from_state(state: AgentState) -> AgentBase:
         config = copy.copy(state.config)
-        config.update({'obs_size': state.obs_space, 'action_size': state.action_space})
+        config.update({'obs_space': state.obs_space, 'action_space': state.action_space})
         agent = DDPGAgent(**config)
         if state.network is not None:
             agent.set_network(state.network)
