@@ -1,5 +1,6 @@
 import copy
 import itertools
+import logging
 from typing import Dict, List
 
 import torch
@@ -16,6 +17,7 @@ from ai_traineree.loggers import DataLogger
 from ai_traineree.networks.bodies import ActorBody
 from ai_traineree.policies import MultivariateGaussianPolicy, MultivariateGaussianPolicySimple
 from ai_traineree.types import ActionType, AgentState, BufferState, DoneType, NetworkState, ObsType, RewardType
+from ai_traineree.types.dataspace import DataSpace
 from ai_traineree.utils import to_numbers_seq, to_tensor
 
 
@@ -30,13 +32,14 @@ class PPOAgent(AgentBase):
         P. Dhariwal, A. Radford, O. Klimov. https://arxiv.org/abs/1707.06347
     """
 
-    name = "PPO"
+    model = "PPO"
+    logger = logging.getLogger("PPO")
 
-    def __init__(self, obs_size: int, action_size: int, **kwargs):
+    def __init__(self, obs_space: DataSpace, action_space: DataSpace, **kwargs):
         """
         Parameters:
-            obs_size (int): Number of input dimensions.
-            action_size (int): Number of output dimensions
+            obs_space (DataSpace): Dataspace describing the input.
+            action_space (DataSpace): Dataspace describing the output.
 
         Keyword parameters:
             hidden_layers (tuple of ints): Shape of the hidden layers in fully connected network. Default: (128, 128).
@@ -63,12 +66,15 @@ class PPOAgent(AgentBase):
         """
         super().__init__(**kwargs)
 
-        self.device = self._register_param(kwargs, "device", DEVICE)  # Default device is CUDA if available
+        self.device = self._register_param(kwargs, "device", DEVICE, update=True)  # Default device is CUDA if available
 
-        self.obs_size = obs_size
-        self.action_size = action_size
-        self._config['obs_size'] = self.obs_size
-        self._config['action_size'] = self.action_size
+        self.obs_space = obs_space
+        self.action_space = action_space
+        assert len(action_space.shape) == 1, "Only 1D actions are supported"
+        self.action_size = action_space.shape[0]
+
+        self._config['obs_space'] = self.obs_space
+        self._config['action_space'] = self.action_space
         self.hidden_layers = to_numbers_seq(self._register_param(kwargs, "hidden_layers", (128, 128)))
         self.iteration = 0
 
@@ -99,9 +105,6 @@ class PPOAgent(AgentBase):
 
         self.local_memory_buffer = {}
 
-        self.action_scale = float(self._register_param(kwargs, "action_scale", 1))
-        self.action_min = float(self._register_param(kwargs, "action_min", -1))
-        self.action_max = float(self._register_param(kwargs, "action_max", 1))
         self.max_grad_norm_actor = float(self._register_param(kwargs, "max_grad_norm_actor", 100.0))
         self.max_grad_norm_critic = float(self._register_param(kwargs, "max_grad_norm_critic", 100.0))
 
@@ -112,10 +115,10 @@ class PPOAgent(AgentBase):
 
         self.buffer = RolloutBuffer(batch_size=self.batch_size, buffer_size=self.rollout_length)
         self.actor = ActorBody(
-            (obs_size,), (self.policy.param_dim*action_size,),
+            self.obs_space.shape, (self.policy.param_dim*self.action_size,),
             gate_out=torch.tanh, hidden_layers=self.hidden_layers, device=self.device)
         self.critic = ActorBody(
-            (obs_size,), (1,),
+            self.obs_space.shape, (1,),
             gate_out=nn.Identity(), hidden_layers=self.hidden_layers, device=self.device)
         self.actor_params = list(self.actor.parameters()) + list(self.policy.parameters())
         self.critic_params = list(self.critic.parameters())
@@ -164,7 +167,7 @@ class PPOAgent(AgentBase):
         actions: List[ActionType] = []
         logprobs = []
         values = []
-        t_obs = to_tensor(obs).view(self.num_workers, self.obs_size).float().to(self.device)
+        t_obs = to_tensor(obs).view((self.num_workers,) + self.obs_space.shape).float().to(self.device)
         for worker in range(self.num_workers):
             actor_est = self.actor.act(t_obs[worker].unsqueeze(0))
             assert not torch.any(torch.isnan(actor_est))
@@ -179,7 +182,6 @@ class PPOAgent(AgentBase):
             if self.is_discrete:  # *Technically* it's the max of Softmax but that's monotonic.
                 action = int(torch.argmax(action))
             else:
-                action = torch.clamp(action*self.action_scale, self.action_min, self.action_max)
                 action = action.cpu().numpy().flatten().tolist()
             actions.append(action)
 
@@ -192,8 +194,8 @@ class PPOAgent(AgentBase):
         self.iteration += 1
 
         self.buffer.add(
-            state=torch.tensor(obs).reshape(self.num_workers, self.obs_size).float(),
-            action=torch.tensor(action).reshape(self.num_workers, self.action_size).float(),
+            state=torch.tensor(obs).reshape((self.num_workers,) + self.obs_space.shape).float(),
+            action=torch.tensor(action).reshape((self.num_workers,) + self.action_space.shape).float(),
             reward=torch.tensor(reward).reshape(self.num_workers, 1),
             done=torch.tensor(done).reshape(self.num_workers, 1),
             logprob=self.local_memory_buffer['logprob'].reshape(self.num_workers, 1),
@@ -216,9 +218,9 @@ class PPOAgent(AgentBase):
         values = to_tensor(experiences['value']).to(self.device)
         logprobs = to_tensor(experiences['logprob']).to(self.device)
         assert rewards.shape == dones.shape == values.shape == logprobs.shape
-        assert states.shape == (self.rollout_length, self.num_workers, self.obs_size), \
+        assert states.shape == (self.rollout_length, self.num_workers) + self.obs_space.shape, \
             f"Wrong states shape: {states.shape}"
-        assert actions.shape == (self.rollout_length, self.num_workers, self.action_size), \
+        assert actions.shape == (self.rollout_length, self.num_workers) + self.action_space.shape, \
             f"Wrong action shape: {actions.shape}"
 
         with torch.no_grad():
@@ -239,19 +241,25 @@ class PPOAgent(AgentBase):
             idx = 0
             self.kl_div = 0
             while idx < self.rollout_length:
-                _states = states[idx:idx+self.batch_size].view(-1, self.obs_size).detach()
-                _actions = actions[idx:idx+self.batch_size].view(-1, self.action_size).detach()
+                _states = states[idx:idx+self.batch_size].view((-1,) + self.obs_space.shape).detach()
+                _actions = actions[idx:idx+self.batch_size].view((-1,) + self.action_space.shape).detach()
                 _logprobs = logprobs[idx:idx+self.batch_size].view(-1, 1).detach()
                 _returns = returns[idx:idx+self.batch_size].view(-1, 1).detach()
                 _advantages = advantages[idx:idx+self.batch_size].view(-1, 1).detach()
                 idx += self.batch_size
                 self.learn((_states, _actions, _logprobs, _returns, _advantages))
 
-            self.kl_div = abs(self.kl_div) / (self.actor_number_updates * self.rollout_length / self.batch_size)
-            if self.kl_div > self.target_kl * 1.75:
-                self.kl_beta = min(2 * self.kl_beta, 1e2)  # Max 100
-            if self.kl_div < self.target_kl / 1.75:
-                self.kl_beta = max(0.5 * self.kl_beta, 1e-6)  # Min 0.000001
+            self.kl_div = abs(self.kl_div) / (self.actor_number_updates * self.num_workers * self.rollout_length / self.batch_size)
+
+            if self.using_kl_div:
+                if self.kl_div > self.target_kl * 1.5:
+                    self.kl_beta = min(1.5 * self.kl_beta, 1e2)  # Max 100
+                elif self.kl_div < self.target_kl / 1.5:
+                    self.kl_beta = max(0.75 * self.kl_beta, 1e-6)  # Min 0.000001
+
+            if self.kl_div > self.target_kl * 1.5:
+                self.logger.warning("Early stopping")
+                break
             self._metrics['policy/kl_beta'] = self.kl_beta
 
     def compute_policy_loss(self, samples):
@@ -295,13 +303,13 @@ class PPOAgent(AgentBase):
     def learn(self, samples):
         self._loss_actor = 0.
 
-        for _ in range(self.actor_number_updates):
+        for actor_iter in range(self.actor_number_updates):
             self.actor_opt.zero_grad()
             loss_actor, kl_div = self.compute_policy_loss(samples)
             self.kl_div += kl_div
             if kl_div > 1.5 * self.target_kl:
                 # Early break
-                # print(f"Iter: {i:02} Early break")
+                self.logger.warning("Early break after %i iterations. %f > %f", actor_iter, kl_div, 1.5*self.target_kl)
                 break
             loss_actor.backward()
             nn.utils.clip_grad_norm_(self.actor_params, self.max_grad_norm_actor)
@@ -340,9 +348,9 @@ class PPOAgent(AgentBase):
 
     def get_state(self) -> AgentState:
         return AgentState(
-            model=self.name,
-            obs_space=self.obs_size,
-            action_space=self.action_size,
+            model=self.model,
+            obs_space=self.obs_space,
+            action_space=self.action_space,
             config=self._config,
             buffer=copy.deepcopy(self.buffer.get_state()),
             network=copy.deepcopy(self.get_network_state())
@@ -366,7 +374,7 @@ class PPOAgent(AgentBase):
     @staticmethod
     def from_state(state: AgentState) -> AgentBase:
         config = copy.copy(state.config)
-        config.update({'obs_size': state.obs_space, 'action_size': state.action_space})
+        config.update({'obs_shape': state.obs_space, 'action_space': state.action_space})
         agent = PPOAgent(**config)
         if state.network is not None:
             agent.set_network(state.network)
