@@ -1,6 +1,7 @@
-# import torch
 from typing import Dict, List, Sequence, Union
 
+
+Numeric = Union[int, float]
 ObsType = ObservationType = Union[List[int], List[float]]
 StateType = Union[int, List[float]]
 ActionType = Union[int, List]
