@@ -1,5 +1,5 @@
 from ai_traineree.agents.dqn import DQNAgent
-from ai_traineree.env_runner import EnvRunner
+from ai_traineree.runners.env_runner import EnvRunner
 from ai_traineree.tasks import GymTask
 
 import pylab as plt
