@@ -1,7 +1,7 @@
 import numpy as np
 import pylab as plt
 
-from ai_traineree.env_runner import EnvRunner
+from ai_traineree.runners.env_runner import EnvRunner
 from ai_traineree.agents.dqn import DQNAgent
 from ai_traineree.tasks import GymTask
 from ai_traineree.types import TaskType
