@@ -2,7 +2,7 @@ import numpy as np
 import pylab as plt
 
 from ai_traineree.agents.td3 import TD3Agent as Agent
-from ai_traineree.env_runner import EnvRunner
+from ai_traineree.runners.env_runner import EnvRunner
 from ai_traineree.tasks import GymTask
 
 
