@@ -4,7 +4,7 @@ import pylab as plt
 from collections import deque
 
 from ai_traineree.agents.dqn import DQNAgent
-from ai_traineree.env_runner import EnvRunner
+from ai_traineree.runners.env_runner import EnvRunner
 from ai_traineree.loggers import TensorboardLogger
 from ai_traineree.networks.heads import NetChainer
 from ai_traineree.networks.bodies import ConvNet, FlattenNet, FcNet, ScaleNet
